<?php

namespace FiloBlu\Refilo\Remote;

use FiloBlu\Refilo\Remote\Entity\EntityProviderInterfaceFactory;
use FiloBlu\Refilo\Remote\Helper\TemplateStringFactory;
use FiloBlu\Refilo\Remote\IndexerConfiguration\IndexerConfigurationRepositoryInterface;
use FiloBlu\Refilo\Remote\IndexerConfiguration\IndexerResolver;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\DataObject;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Store\Api\Data\StoreInterface;
use Magento\Store\Model\StoreManagerInterface;
use Psr\Log\LoggerInterface;

/**
 * Class AbstractWebsiteBasedIndexer
 * @package FiloBlu\Refilo\Remote
 */
abstract class AbstractWebsiteBasedIndexer extends AbstractIndexer
{
    /** @var string */
    const ARGUMENT_WEBSITE = 'website';

    /**
     * @param array $ids
     * @return mixed|void
     * @throws NoSuchEntityException
     */
    public function executeAction(array $ids)
    {
        try {
            $indexTemplate = $this->getGetRemoteIndexTemplate();
        } catch (\Exception $exception) {
            $this->logger->error($exception->getMessage(), ['exception' => $exception]);
            return;
        }

        foreach ($this->getStoreManager()->getWebsites() as $website) {

            $arguments = $this->dataObjectFactory->create([
                'data' => [
                    self::ARGUMENT_WEBSITE => $website
                ]
            ]);

            iterator_to_array($this->entityProviderFactory->create()
                ->forWebsite($website)
                ->withIds($ids)
                ->attachReaderHandler($this, $arguments));
        }
    }

    /**
     * @param $entities
     * @param $size
     * @param DataObject $arguments
     * @throws NoSuchEntityException
     */
    public function onRead($entities, $size, DataObject $arguments)
    {
        $website = $arguments->getData(self::ARGUMENT_WEBSITE);
        $previousStore = $this->getStoreManager()->getStore();

        /** @var StoreInterface $store */
        foreach ($website->getStores() as $store) {
            $this->getStoreManager()->setCurrentStore($store);

            if (!$store->getIsActive()) {
                continue;
            }

            $collection = $this->getGetRemoteIndexTemplate()->withValue('%s', $store->getCode())->render();

            parent::onRead($entities, $size, $this->dataObjectFactory->create([
                'data' => [
                    self::ARGUMENT_COLLECTION => $collection
                ]
            ]));

        }

        $this->getStoreManager()->setCurrentStore($previousStore);
    }
}
